if select(2, UnitClass("player")) ~= "PALADIN" then return end

local Module = UnderHood:GetModule("HolyPower")
local L = LibStub("AceLocale-3.0"):GetLocale("UnderHood_Special")

local ipairs = ipairs

local HolyPowerFrame = UnderHood:RegisterFrameClass("HolyPower", L["HolyPower"])

local runeDescriptors = "Interface\\PlayerFrame\\PaladinPowerTextures"

local layouts = {
	D = {
		size = function(size, space) return size, size * 5 + space * 4 end,
		point = "TOP", relative = "BOTTOM", dx = 0, dy = -1,
	},
	U = {
		size = function(size, space) return size, size * 5 + space * 4 end,
		point = "BOTTOM", relative = "TOP", dx = 0, dy = 1,
	},
	L = {
		size = function(size, space) return size * 5 + space * 4, size end,
		point = "RIGHT", relative = "LEFT", dx = -1, dy = 0,
	},
	R = {
		size = function(size, space) return size * 5 + space * 4, size end,
		point = "LEFT", relative = "RIGHT", dx = 1, dy = 0,
	},
}

function HolyPowerFrame:init()
	super(self)

	self.runes = {}

	for i = 1, 3 do
		local rune = {}

		rune.texture = self.frame:CreateTexture()
		rune.texture:Show()

		self.runes[i] = rune
	end

	LibStub("AceEvent-3.0"):Embed(self)
end

function HolyPowerFrame:OnAcquire(settings)
	super(self, settings)

	self.frame:Show()

	self:RegisterEvent("UNIT_POWER", "UpdateRunes")

	self:PerformLayout()
	self:UpdateRunes()
end

function HolyPowerFrame:OnRelease()
	self:UnregisterAllEvents()

	super(self)
end

function HolyPowerFrame:GetDefaultSettings()
	local settings = super(self)

	settings.positionAndSize.width = 1
	settings.positionAndSize.height = 1

	settings.runes = {
		layout = "D",
		size = 16,
		spacing = 0,
		style = "default",
	}

	return settings
end

--[[
function HolyPowerFrame:ConfigurationModeChanged(mode)
	super(self, mode)

	self:UpdateRunes()
end
--]]

function HolyPowerFrame:PerformLayout()
	local settings = self.settings.runes
	local layout = layouts[settings.layout]
	local size = settings.size
	local spacing = settings.spacing
	local width, height = layout.size(size, spacing)

	self.frame:SetWidth(width)
	self.frame:SetHeight(height)

	for i, rune in ipairs(self.runes) do
		local tx = rune.texture

		tx:SetWidth(size)
		tx:SetHeight(size)
		tx:ClearAllPoints()

		if i == 1 then
			tx:SetPoint(layout.point, self.frame, layout.point, 0, 0)
		else
			tx:SetPoint(layout.point, self.runes[i-1].texture, layout.relative, spacing * layout.dx, spacing * layout.dy)
		end
	end
end

function HolyPowerFrame:UpdateRunes(event, unit, powertype)
	if powertype ~= SPELL_POWER_HOLY_POWER then return end

	local numHolyPower = UnitPower("player", powertype);
	for i = 1, MAX_HOLY_POWER do
		local holyRune = self.runes[i]
		local shouldShow = i <= numHolyPower
		if shouldShow then
			holyRune:Show()
		else
			holyRune:Hide()
		end
	end

--[[if numHolyPower == MAX_HOLY_POWER then
		self.glow.pulse.stopPulse = false;
		self.glow.pulse:Play();
	else
		self.glow.pulse.stopPulse = true;
	end]]
end

local validLayouts = {
	D = L["Down"],
	U = L["Up"],
	R = L["Right"],
	L = L["Left"],
}

function HolyPowerFrame:CreateOptions()
	local options = super(self)

	options.positionAndSize.args.width = nil
	options.positionAndSize.args.height = nil

	options.positionAndSize.args.h2 = {
		type = "header",
		name = L["HolyPower"],
		order = 100,
	}

	options.positionAndSize.args.layout = {
		name = L["Layout"],
		type = "select",
		order = 110,
		values = validLayouts,
		get = function() return self.settings.runes.layout end,
		set = function(info, value)
			self.settings.runes.layout = value
			self:PerformLayout()
		end,
	}

	options.positionAndSize.args.size = {
		name = L["Rune size"],
		type = "range",
		order = 120,
		min = 10,
		max = 48,
		step = 1,
		get = function() return self.settings.runes.size end,
		set = function(info, value)
			self.settings.runes.size = value
			self:PerformLayout()
		end,
	}

	options.positionAndSize.args.spacing = {
		name = L["Space between runes"],
		type = "range",
		order = 150,
		min = 0,
		max = 40,
		step = 1,
		get = function() return self.settings.runes.spacing end,
		set = function(info, value)
			self.settings.runes.spacing = value
			self:PerformLayout()
		end,
	}

	return options
end
