if select(2, UnitClass("player")) ~= "WARLOCK" then return end

local Module = UnderHood:GetModule("SoulShards")
local L = LibStub("AceLocale-3.0"):GetLocale("UnderHood_Special")

local ipairs = ipairs

local ShardsFrame = UnderHood:RegisterFrameClass("SoulShards", L["SoulShards"])

local shardDescriptors = "Interface\\PlayerFrame\\UI-WarlockShard"
local SPELL_POWER_SOUL_SHARDS = SPELL_POWER_SOUL_SHARDS

local layouts = {
	D = {
		size = function(size, space) return size, size * 5 + space * 4 end,
		point = "TOP", relative = "BOTTOM", dx = 0, dy = -1,
	},
	U = {
		size = function(size, space) return size, size * 5 + space * 4 end,
		point = "BOTTOM", relative = "TOP", dx = 0, dy = 1,
	},
	L = {
		size = function(size, space) return size * 5 + space * 4, size end,
		point = "RIGHT", relative = "LEFT", dx = -1, dy = 0,
	},
	R = {
		size = function(size, space) return size * 5 + space * 4, size end,
		point = "LEFT", relative = "RIGHT", dx = 1, dy = 0,
	},
}

function ShardsFrame:init()
	super(self)

	self.shards = {}

	for i = 1, 3 do
		local shard = {}

		shard.texture = self.frame:CreateTexture()
		shard.texture:SetTexture(descr[2])
		shard.texture:Show()

		self.shards[i] = shard
	end

	LibStub("AceEvent-3.0"):Embed(self)
end

function ShardsFrame:OnAcquire(settings)
	super(self, settings)

	self.frame:Show()

	self:RegisterEvent("UNIT_POWER", "UpdateShards")

	self:PerformLayout()
	self:UpdateShards()
end

function ShardsFrame:OnRelease()
	self:UnregisterAllEvents()

	super(self)
end

function ShardsFrame:GetDefaultSettings()
	local settings = super(self)

	settings.positionAndSize.width = 1
	settings.positionAndSize.height = 1

	settings.shards = {
		layout = "D",
		size = 16,
		spacing = 0,
		style = "default",
	}

	return settings
end

--[[
function ShardsFrame:ConfigurationModeChanged(mode)
	super(self, mode)

	self:UpdateShards()
end
--]]

function ShardsFrame:PerformLayout()
	local settings = self.settings.shards
	local layout = layouts[settings.layout]
	local size = settings.size
	local spacing = settings.spacing
	local width, height = layout.size(size, spacing)

	self.frame:SetWidth(width)
	self.frame:SetHeight(height)

	for i, shard in ipairs(self.shards) do
		local tx = shard.texture

		tx:SetWidth(size)
		tx:SetHeight(size)
		tx:ClearAllPoints()

		if i == 1 then
			tx:SetPoint(layout.point, self.frame, layout.point, 0, 0)
		else
			tx:SetPoint(layout.point, self.shards[i-1].texture, layout.relative, spacing * layout.dx, spacing * layout.dy)
		end
	end
end

function ShardsFrame:UpdateShards(event, unit, powertype)
	if powertype ~= SPELL_POWER_SOUL_SHARDS then return end

	local numShards = UnitPower("player", powertype)
	for i = 1, SHARD_BAR_NUM_SHARDS do
		local shard = self.shards[i]

		local shouldShow = i <= numShards;
		if shouldShow then
			shard:Show()
		else
			shard:Hide()
		end
	end
end

local validLayouts = {
	D = L["Down"],
	U = L["Up"],
	R = L["Right"],
	L = L["Left"],
}

function ShardsFrame:CreateOptions()
	local options = super(self)

	options.positionAndSize.args.width = nil
	options.positionAndSize.args.height = nil

	options.positionAndSize.args.h2 = {
		type = "header",
		name = L["Shards"],
		order = 100,
	}

	options.positionAndSize.args.layout = {
		name = L["Layout"],
		type = "select",
		order = 110,
		values = validLayouts,
		get = function() return self.settings.shards.layout end,
		set = function(info, value)
			self.settings.shards.layout = value
			self:PerformLayout()
		end,
	}

	options.positionAndSize.args.size = {
		name = L["Shard size"],
		type = "range",
		order = 120,
		min = 10,
		max = 48,
		step = 1,
		get = function() return self.settings.shards.size end,
		set = function(info, value)
			self.settings.shards.size = value
			self:PerformLayout()
		end,
	}

	options.positionAndSize.args.spacing = {
		name = L["Space between shards"],
		type = "range",
		order = 150,
		min = 0,
		max = 40,
		step = 1,
		get = function() return self.settings.shards.spacing end,
		set = function(info, value)
			self.settings.shards.spacing = value
			self:PerformLayout()
		end,
	}

	return options
end
